
/** Cette classe propose une implementation des arcs orientes
 * pouvant figurer dans un {@link GrapheOriente} et un ensemble
 * de methodes permettant de travailler sur ces arcs
 */
public class ArcOriente  {
	protected Sommet depart;
	protected Sommet arrivee;

	/** Un constructeur d'arc :
	 * @param from : le {@link Sommet} d'origine de l'arc
	 * @param to : le {@link Sommet} d'arrivee de l'arc
	 */
	public ArcOriente(Sommet from, Sommet to)
	{
		depart = from;
		arrivee = to;
	}

	/** Recupere le sommet de depart d'un ArcOriente.
	 * @return Un {@link Sommet}
	 */
	public Sommet getDepart() {
		return depart;
	}

	/** Recupere le sommet d'arrivee d'un ArcOriente
	 * @return : Un {@link Sommet}
	 */
	public Sommet getArrivee() {
		return arrivee;
	}

	/** Teste si l'arc arrive en un sommet donne.
	 * @param s : le sommet a tester
	 * @return : Un boolean, true si le sommet arrive en s, false sinon
	 */
	public boolean arriveEn(Sommet s) {
		if (s.equals(arrivee))
			return true;
		return false;
	}

	/** Teste si l'arc part d'un sommet donne.
	 * @param s : le sommet a tester
	 * @return : Un boolean, true si le sommet part de s, false sinon
	 */
	public boolean partDe(Sommet s) {
		if (s.equals(depart))
			return true;
		return false;
	}

	/** Teste si l'arc courant est egal a un autre ArcOriente.
	 * Deux ArcsOriente sont consideres egaux si leur sommet de depart
	 * et leur sommet d'arrivee sont les memes.
	 * @param a : l'arc a tester.
	 * @return : Un boolean, true si les arcs sont identiques, false sinon
	 */
	public boolean equals(ArcOriente a) {
		return (depart.equals(a.getDepart()) && arrivee.equals(a.getArrivee()) );
	}

	/** Teste si l'arc courant est egal a un autre Objet
	 * Si l'objet est un ArcOriente, cette methode teste l'egalite entre ArcOrientes
	 * Si l'objet n'est pas un ArcOriente, cette methode retourne false.
	 * @param o : l'arc a tester.
	 * @return : Un boolean. 
	 */
	public boolean equals (Object o) {
		if (o instanceof ArcOriente)
			return equals((ArcOriente) o);
		return false;
	}

}

