
/** Cette classe implemente des arcs orientes values
 * pour l'utilisation dans des {@link GrapheOrienteValue}.
 *
 * Un ArcOrienteValue est un {@link ArcOriente} auquel on ajoute un poid.
 * Ce poid peut par exemple correspondre a la longueur de l'arete
 * et est du type double.
 * */
public class ArcOrienteValue extends ArcOriente implements Comparable {
	protected double poid;

	/** Un constructeur :
	 * @param from le {@link Sommet} de depart
	 * @param to le {@link Sommet} d'arrivee
	 * @param poid le poid de l'arete
	 * */
	public ArcOrienteValue(Sommet from, Sommet to, double poid)
	{
		super(from, to);
		this.poid = poid;
	}

	/** Connaitre le poid d'une arete.
	 * @return un double correspondant au poid de l'arete.
	 */
	public double getPoids() {
		return poid;
	}

	/** Teste l'egalite entre l'arc courant et un
	 * autre ArcOrienteValue.
	 * Deux ArcOriente sont consideres egaux si leurs sommets de depart
	 * sont identiques et leurs sommets d'arrivee sont identiques.
	 * @return un boolean
	 */
	public boolean equals(ArcOrienteValue a) {
		return (depart.equals(a.getDepart()) && arrivee.equals(a.getArrivee()) );
	}

	public boolean equals (Object o) {
		if (o instanceof ArcOrienteValue)
			return equals((ArcOrienteValue) o);
		if (o instanceof ArcOriente)
			return super.equals((ArcOriente) o);
		return false;
	}

	/** Compare deux ArcsOrientes.
	 * Permet de definir un ordre sur un ensemble d'arcs orientes.
	 * l'ordre propose par cette methode est l'ordre des poids croissants :
	 * @param autreArc : un ArcOriente a tester face a l'arc courant.
	 * @return 1  si le poid de l'arc courant est plus grand que l'arc teste.
	 * @return -1 si le poid de l'arc courant est plus faible que l'arc teste.
	 * @return 0  si les poids des deux arcs sont egaux.
	 */
	public int compareTo (ArcOrienteValue autreArc) {
		double poid2 = autreArc.getPoids();
		if (poid < poid2)
			return -1;
		if (poid > poid2)
			return 1;
		return 0;
	}

        public int compareTo (Object o) {
	      	if  ( !(o instanceof ArcOrienteValue) )
    			throw new IllegalArgumentException("On compare un noeud a autre chose !");
					               
		return compareTo((ArcOrienteValue) o);
	}

}

