import java.util.*;

/** Cette classe propose une implementation d'un chemin entre sommets.
 * Il s'agit en fait d'une Liste d'arcs Orientes passant par des sommets successifs.
 */
public class Chemin extends LinkedList {

	public Chemin()
	{
		super();
	}

	/** ajoute un ArcOriente a la fin d'un chemin.
	 * Si le Sommet de depart de l'arc a ajouter n'est
	 * pas egal au Sommet d'arrivee du dernier arc du chemin
	 * une exception est levee.
	 */
	public void addFin(ArcOriente arc) {
		Sommet depart = arc.getDepart();
		Sommet arrivee = arc.getArrivee();
		if (super.isEmpty()) {
			addFirst(arc);
			return;
		}

		Sommet arriveeActuelle = getArrivee();
		if (!depart.equals(arriveeActuelle))
			throw new IllegalArgumentException ("Les Sommets ne se succedent pas d'arc en arc");
		super.addLast(arc);
	}

	/** ajoute un ArcOriente a la fin d'un chemin.
	 * Si le Sommet de depart de l'arc a ajouter n'est
	 * pas egal au Sommet d'arrivee du dernier arc du chemin
	 * une exception est levee.
	 */
	public void addDebut(ArcOriente arc) {
		Sommet depart = arc.getDepart();
		Sommet arrivee = arc.getArrivee();
		if (super.isEmpty()) {
			addFirst(arc);
			return;
		}

		Sommet departActuel = getDepart();
		if (!arrivee.equals(departActuel))
			throw new IllegalArgumentException ("Les Sommets ne se succedent pas d'arc en arc");
		super.addFirst(arc);
		/*
		System.out.println("Ajout d'un arc "+ ((SommetLabelise)depart).getLabel()+ " "
					+ ((SommetLabelise)arrivee).getLabel());
		*/
	}

	/** Renvoie le sommet de depart d'un chemin.
	 * Une exception est levee si le chemin est vide
	 */
	public Sommet getDepart() {
		ArcOriente arc;
		if (super.isEmpty())
			throw new IllegalArgumentException ("Le chemin est vide");
		arc =  (ArcOriente) super.getFirst();
		return arc.getDepart();
	}

	/** Renvoie le sommet d'arrivee d'un chemin.
	 * Une exception est levee si le chemin est vide
	 */
	public Sommet getArrivee() {
		ArcOriente arc;
		if (isEmpty())
			throw new IllegalArgumentException ("Le chemin est vide");
		arc =  (ArcOriente) super.getLast();
		return arc.getArrivee();
	}


	/** Renvoie l'arc d'arrivee d'un chemin.
	 * Une exception est levee si le chemin est vide.
	 */
	public ArcOriente getArcArrivee() {
		ArcOriente arc;
		if (super.isEmpty())
			throw new IllegalArgumentException ("Le chemin est vide");
		return (ArcOriente) super.getLast();
	}

	/** Renvoie une liste des sommets du chemin.
	 * Dans le cas ou le chemin est vide, la liste la liste renvoyée est vide.
	 */
	public LinkedList getSommets() {
		LinkedList l = new LinkedList();
		ArcOriente arcCourant ;
		Sommet courant,arrivee ;
		//System.out.println("Les Sommets du chemin :");

		if (isEmpty())
			return l;
		
		Iterator iter=this.iterator();

		while (iter.hasNext()) {
			arcCourant = (ArcOriente) iter.next();
			courant = arcCourant.getDepart();
			arrivee = arcCourant.getArrivee();
		//	System.out.println("Sommet : "+((SommetLabelise)courant).getLabel());
		//	System.out.println("Arc : "+((SommetLabelise)courant).getLabel()+" " +((SommetLabelise)arrivee).getLabel());
			l.addLast(courant);
		}
		l.addLast(this.getArrivee());

		return l;
	}

	/** Renvoie la longueur du chemin.
	 * Cette longueur est le nombre d'aretes du chemin, pas le nombre de sommet.
	 * Le nombre de sommets est obtenu facilement : il est egal a la longueur +1.
	 */
	public int longueur() {
		return this.size();
	}

	/** Renvoie une String contenant les labels concatenes de tous les sommets.
	 * Cette methode devrait disparaitre par la suite, elle ne sert pas a grand chose.
	 */
	public String labelSommets() {
		if (isEmpty())
			return "Chemin Vide";
		
		Sommet depart = getDepart();
		Sommet arrivee = getArrivee();
		SommetLabelise courant;
		ArcOriente arcCourant;

		String s =new String();
	
		if ( (depart instanceof SommetLabelise) && (arrivee instanceof SommetLabelise))
			s += "Chemin depart en "
				+ ((SommetLabelise)depart).getLabel()
				+ " fin en " +((SommetLabelise)arrivee).getLabel()+"\n";
		else
			throw new IllegalArgumentException("Ceci ne fonctionne qu'avec des sommets labelises !");
		Iterator iter = this.iterator();

		while (iter.hasNext()) {
			arcCourant = (ArcOriente) iter.next();
			courant = (SommetLabelise) arcCourant.getDepart();
			s+=courant.getLabel();
			s+=" / ";
		}
	
		courant = (SommetLabelise) arrivee;
		s+=courant.getLabel();
		s+="\n";
		
		return s;
	}

	/* Pour empecher les construction n'utilisant pas les methodes
	 * de cette classe, il faudra surcharger TOUTES les methodes
	 * de la classe LinkedList...
	 */
	/** Ne pas appeler cette méthode.
	 * Il s'agit d'une surcharge de methode pour eviter les problemes.
	 * Elle lancera necessairement une Exception si appelee.
	 */
	 public boolean add(Object o) {
		throw new IllegalArgumentException ("Dans un chemin, on ajoute des arcs orientes");
	 }
}

