
import java.io.*;
import java.util.*;

/** Cette classe imlemente les Reseaux de transport.
 * Un reseau de transport est un graphe oriente dans lequel on distingue
 * deux sommets particuliers, la source et le puit.
 * La valeur d'une arete est strictement positive
 *<p>
 * Cette classe n'utilise pas la notion de Generic 
 * car le JDK des salles de TP ne le supporte pas.
 * Pour une version plus recente, demandez !
 */
public class ReseauTransport extends GrapheOrienteValue {
	Sommet source;
	Sommet puit;

	/** Construit un reseau de transport en precisant la source et le puit.
	 */
	public ReseauTransport (Sommet s, Sommet p) {
		super();
		addSommet(s);
		addSommet(p);
		source = s;
		puit = p;
	}


	/** Renvoie la capacite d'un arc
	 */
	public double getCapacite(ArcOrienteValue arc) {
		return arc.getPoids();
	}

	/** Renvoie la capacite d'un arc defini par ses sommets
	 */
	public double getCapacite(Sommet depart, Sommet fin) {
		return getArc(depart,fin).getPoids();
	}

	/** Renvoie le sommet source du Reseau de Transport
	 */
	public Sommet getSource() {
		return source;
	}

	/** Renvoie le sommet puit du Reseau de Transport
	 */
	public Sommet getPuit() {
		return puit;
	}

	/** Ajoute un arc au Reseau.
	 * Si la capacite de l'arc est negative ou nulle,
	 * l'arc n'est pas ajoute
	 */
	public void addArc(ArcOrienteValue arc) {
		if (arc.getPoids() >0 )
			super.addArc(arc);
	}

	/** Ajoute un arc au Reseau.
	 * Si la capacite de l'arc est negative ou nulle,
	 * l'arc n'est pas ajoute
	 */
	public void addArc(Sommet depart, Sommet fin, double capacite) {
		if (capacite >0 )
			super.addArc(depart,fin, capacite);
	}

}

